@@ -113,8 +113,8 @@ int  xine_check_version (int major, int minor, int sub) XINE_PROTECTED;
 /* static info - which libxine release this header came from */
 #define XINE_MAJOR_VERSION 1
 #define XINE_MINOR_VERSION 1
-#define XINE_SUB_VERSION   15
-#define XINE_VERSION       "1.1.15"
+#define XINE_SUB_VERSION   16
+#define XINE_VERSION       "1.1.16"
 
 /*
  * pre-init the xine engine
@@ -993,6 +993,7 @@ const char *xine_get_meta_info   (xine_stream_t *stream, int info) XINE_PROTECTE
 #define XINE_META_INFO_INPUT_PLUGIN        9
 #define XINE_META_INFO_CDINDEX_DISCID      10
 #define XINE_META_INFO_TRACK_NUMBER        11
+#define XINE_META_INFO_COMPOSER            12
 
 
 /*********************************************************************
@@ -1916,7 +1917,7 @@ typedef struct {
 typedef struct {
   int                 alternative; /* alternative playlist number, usually 0 */
   char                mrl[1]; /* might (will) be longer */
-} xine_mrl_reference_data_t;
+} xine_mrl_reference_data_t XINE_DEPRECATED;
 
 typedef struct {
   int                 alternative; /* as above */
