@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2007 the xine project
+ * Copyright (C) 2001-2008 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -78,6 +78,9 @@ struct ff_video_decoder_s {
 
   xine_stream_t    *stream;
   int64_t           pts;
+  uint64_t          pts_tag_mask;
+  uint64_t          pts_tag;
+  int               pts_tag_counter;
   int               video_step;
 
   uint8_t           decoder_ok:1;
@@ -606,6 +609,10 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
   su = this->av_frame->data[1];
   sv = this->av_frame->data[2];
 
+  /* Some segfaults & heap corruption have been observed with img->height,
+   * so we use this->bih.biHeight instead (which is the displayed height)
+   */
+
   if (this->context->pix_fmt == PIX_FMT_YUV410P) {
 
     yuv9_to_yv12(
@@ -626,7 +633,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
       img->pitches[2],
      /* width x height */
       img->width,
-      img->height);
+      this->bih.biHeight);
 
   } else if (this->context->pix_fmt == PIX_FMT_YUV411P) {
 
@@ -648,7 +655,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
       img->pitches[2],
      /* width x height */
       img->width,
-      img->height);
+      this->bih.biHeight);
 
   } else if (this->context->pix_fmt == PIX_FMT_RGBA32) {
           
@@ -656,7 +663,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
     uint32_t *argb_pixels;
     uint32_t argb;
 
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       argb_pixels = (uint32_t *)sy;
       for(x = 0; x < img->width; x++) {
         uint8_t r, g, b;
@@ -684,7 +691,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
     uint8_t *src;
     uint16_t pixel16;
 
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       src = sy;
       for(x = 0; x < img->width; x++) {
         uint8_t r, g, b;
@@ -713,7 +720,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
     uint8_t *src;
     uint16_t pixel16;
             
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       src = sy;
       for(x = 0; x < img->width; x++) {
         uint8_t r, g, b;
@@ -741,7 +748,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
     int x, plane_ptr = 0;
     uint8_t *src;
 
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       src = sy;
       for(x = 0; x < img->width; x++) {
         uint8_t r, g, b;
@@ -765,7 +772,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
     int x, plane_ptr = 0;
     uint8_t *src;
 
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       src = sy;
       for(x = 0; x < img->width; x++) {
         uint8_t r, g, b;
@@ -808,7 +815,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
       v_palette[x] = COMPUTE_V(r, g, b);
     }
 
-    for(y = 0; y < img->height; y++) {
+    for(y = 0; y < this->bih.biHeight; y++) {
       src = sy;
       for(x = 0; x < img->width; x++) {
         pixel = *src++;
@@ -825,7 +832,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
           
   } else {
           
-    for (y=0; y<img->height; y++) {
+    for (y = 0; y < this->bih.biHeight; y++) {
       xine_fast_memcpy (dy, sy, img->width);
   
       dy += img->pitches[0];
@@ -833,7 +840,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
       sy += this->av_frame->linesize[0];
     }
 
-    for (y=0; y<(img->height/2); y++) {
+    for (y = 0; y < this->bih.biHeight / 2; y++) {
       
       if (this->context->pix_fmt != PIX_FMT_YUV444P) {
         
@@ -1174,6 +1181,42 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
   }
 }
 
+static uint64_t ff_tag_pts(ff_video_decoder_t *this, uint64_t pts)
+{
+  return pts | this->pts_tag;
+}
+
+static uint64_t ff_untag_pts(ff_video_decoder_t *this, uint64_t pts)
+{
+  if (this->pts_tag_mask == 0)
+    return pts; /* pts tagging inactive */
+
+  if (this->pts_tag != 0 && (pts & this->pts_tag_mask) != this->pts_tag)
+    return 0; /* reset pts if outdated while waiting for first pass (see below) */
+
+  return pts & ~this->pts_tag_mask;
+}
+
+static void ff_check_pts_tagging(ff_video_decoder_t *this, uint64_t pts)
+{
+  if (this->pts_tag_mask == 0)
+    return; /* pts tagging inactive */
+
+  if ((pts & this->pts_tag_mask) != this->pts_tag)
+    return; /* pts still outdated */
+
+  if (this->pts_tag != 0) {
+    /* first pass: reset pts_tag */
+    this->pts_tag = 0;
+  } else if (pts == 0)
+    return; /* cannot detect second pass */
+  else {
+    /* second pass: reset pts_tag_mask and pts_tag_counter */
+    this->pts_tag_mask = 0;
+    this->pts_tag_counter = 0;
+  }
+}
+
 static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
   uint8_t *chunk_buf = this->buf;
   AVRational avr00 = {0, 1};
@@ -1198,6 +1241,13 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
     this->size = 0;
   }
 
+  if (this->size == 0) {
+    /* take over pts when we are about to buffer a frame */
+    this->av_frame->reordered_opaque = ff_tag_pts(this, this->pts);
+    this->context->reordered_opaque = ff_tag_pts(this, this->pts);
+    this->pts = 0;
+  }
+
   /* data accumulation */
   if (buf->size > 0) {
     if ((this->size == 0) &&
@@ -1250,6 +1300,10 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         len = avcodec_decode_video (this->context, this->av_frame,
                                     &got_picture, &chunk_buf[offset],
                                     this->size);
+
+        /* reset consumed pts value */
+        this->context->reordered_opaque = ff_tag_pts(this, 0);
+
         lprintf("consumed size: %d, got_picture: %d\n", len, got_picture);
         if ((len <= 0) || (len > this->size)) {
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
@@ -1265,6 +1319,11 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
             ff_check_bufsize(this, this->size);
             memmove (this->buf, &chunk_buf[offset], this->size);
             chunk_buf = this->buf;
+
+            /* take over pts for next access unit */
+            this->av_frame->reordered_opaque = ff_tag_pts(this, this->pts);
+            this->context->reordered_opaque = ff_tag_pts(this, this->pts);
+            this->pts = 0;
           }
         }
       }
@@ -1359,8 +1418,9 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
           ff_convert_frame(this, img);
         }
 
-        img->pts  = this->pts;
-        this->pts = 0;
+        img->pts  = ff_untag_pts(this, this->av_frame->reordered_opaque);
+        ff_check_pts_tagging(this, this->av_frame->reordered_opaque); /* only check for valid frames */
+        this->av_frame->reordered_opaque = 0;
 
         /* workaround for weird 120fps streams */
         if( video_step_to_use == 750 ) {
@@ -1400,8 +1460,8 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
                                                 this->output_format,
                                                 VO_BOTH_FIELDS|this->frame_flags);
       /* set PTS to allow early syncing */
-      img->pts       = this->pts;
-      this->pts      = 0;
+      img->pts       = ff_untag_pts(this, this->av_frame->reordered_opaque);
+      this->av_frame->reordered_opaque = 0;
 
       img->duration  = video_step_to_use;
 
@@ -1486,6 +1546,10 @@ static void ff_reset (video_decoder_t *this_gen) {
   
   if (this->is_mpeg12)
     mpeg_parser_reset(this->mpeg_parser);
+
+  this->pts_tag_mask = 0;
+  this->pts_tag = 0;
+  this->pts_tag_counter = 0;
 }
 
 static void ff_discontinuity (video_decoder_t *this_gen) {
@@ -1493,6 +1557,37 @@ static void ff_discontinuity (video_decoder_t *this_gen) {
   
   lprintf ("ff_discontinuity\n");
   this->pts = 0;
+
+  /*
+   * there is currently no way to reset all the pts which are stored in the decoder.
+   * therefore, we add a unique tag (generated from pts_tag_counter) to pts (see 
+   * ff_tag_pts()) and wait for it to appear on returned frames.
+   * until then, any retrieved pts value will be reset to 0 (see ff_untag_pts()).
+   * when we see the tag returned, pts_tag will be reset to 0. from now on, any
+   * untagged pts value is valid already.
+   * when tag 0 appears too, there are no tags left in the decoder so pts_tag_mask
+   * and pts_tag_counter will be reset to 0 too (see ff_check_pts_tagging()).
+   */
+  this->pts_tag_counter++;
+  this->pts_tag_mask = 0;
+  this->pts_tag = 0;
+  {
+    /* pts values typically don't use the uppermost bits. therefore we put the tag there */
+    int counter_mask = 1;
+    uint64_t tag_mask = 0x8000000000000000ull;
+    while (this->pts_tag_counter >= counter_mask)
+    {
+      /*
+       * mirror the counter into the uppermost bits. this allows us to enlarge mask as
+       * necessary and while previous taggings can still be detected to be outdated.
+       */
+      if (this->pts_tag_counter & counter_mask)
+        this->pts_tag |= tag_mask;
+      this->pts_tag_mask |= tag_mask;
+      tag_mask >>= 1;
+      counter_mask <<= 1;
+    }
+  }
 }
 
 static void ff_dispose (video_decoder_t *this_gen) {
@@ -1530,10 +1625,10 @@ static void ff_dispose (video_decoder_t *this_gen) {
     free_yuv_planes(&this->yuv);
   
   if( this->context )
-    free( this->context );
+    av_free( this->context );
 
   if( this->av_frame )
-    free( this->av_frame );
+    av_free( this->av_frame );
   
   if (this->buf)
     free(this->buf);
